*** Outline

* # 7.2 Artificial data for the Human Capital model
* # 7.2.1 Definitions and artificial data generation
* # 7.2.2 Heteroskedasticity
* # 7.2.3 Errors in variables
* # 7.2.4 Splines
* # 7.2.5 Instrumental variables
* # 7.2.6 Wage gap decomposition

version 12.1
clear
set more off

cd "T:\Labor_economics\ES\ES5\Datasets_HCM"

* #######################################################
* ### 7.2 Artificial data for the Human Capital model ###

*** Usefull commands for the session
* gen rnd=rnormal()   			/* Generates normaly distributed variable rnd */
* mat B=el(matrix(P),1,10)		/* Retrieves element 1,10 from matrix P */
* drawnorm 						/* Provides an alternative way to generate multiple normal variables, 
								/* and optionally to specify the correlations between them */

* ########################################################
* ### 7.2.1 Definitions and artificial data generation ###

/* 1. Create artificial dataset of earnings, age, education for 2300 independent individuals 
according to the Mincerian Human Capital Model presented as:

logEARN=a+b1*EDU+c1*EXP+c2*EXP^2+e, where EXP=AGE-EDU-6. 

The deterministic part of this equation is called underlying model.

To create artificial dataset we need to:
# a. Generate right hand side variables. 
Variables should have plausible (in line with empirical evidence) range of values, 
distributions and should have reasonable V-C matrix (reflecting likely correlations of RHS variables).
# b. Set reasonable coefficients a, b1, c1, and c2, 
generate stochastic term e of plausible iid, and generate left-hand side variable.

# c. Estimate underlying model by OLS using underlying functional form. */

*DEFINITIONS

clear
mat in m=(12,20,0)	/* matrix of means of RHS variables: edu, exp, error */
mat in c=(5,-.6, 0 \ -.6,119,0 \ 0,0,.1) /* covariance matrix of RHS variables */

matrix list m	/* displays matrix of means */
matrix list c	/* displays covariance matrix */

set seed 12345 // Specify initial value of random-number seed

* Draw a sample of 2300 observations from a multivariate normal distribution with desired means and covariance matrix.

drawnorm edu exp e, n(2300) means(m) cov(c)   

 *HINT: make sure you understand where and why we set zero correlation between error and regressors

gen exp2=exp^2					 				

* Generate log of earnings

gen logY = 7.6 + edu*.07 + exp*.012 - exp2*.0005 + e

drop if exp<0 | exp>40	/* Dropping observations with extreme values of exp */

correlate edu exp e, cov m  /* Compare means and covariance matrix for generated data and parameters you required */

* SOME REGRESSIONS

reg logY edu exp				/* See that you get negative exp coefficient if omitting exp2 */
reg logY edu exp exp2			/* Right specification */

// If relevant variables are omitted from the model, the common variance they share with included variables 
// may be wrongly attributed to those variables, and the error term is inflated. 

*** Check for error in model specification

linktest
ovtest

* No error in model specification

*** Check for multicollinearity

vif
collin edu exp exp2, corr

* Condition Numbers (CN)
* # If (CN < 100)			Weak
* # If (100< CN < 1000)		Moderate to Strong
* # If (CN > 1000) 			Severe

* Condition Index (CI)
* # If ((CI < 10)			Weak
* # If ((10 < CI < 30)		Moderate to Strong
* # If (CI > 30) 			Severe

// Since CN<100 and CI<10 we face week multicollinearity

*** Checking Linearity

gen Y=exp(logY)

graph matrix Y edu exp exp2
acprplot edu, lowess
acprplot exp, lowess // exp^2 is collinear with exp
acprplot exp2, lowess

graph matrix logY edu exp exp2
acprplot edu, lowess
acprplot exp, lowess // exp^2 is collinear with exp
acprplot exp2, lowess

* Compare normal and lognormal distribution

// gen Y=exp(logY)

graph box Y, saving(box_Y, replace)
graph box logY, saving(box_logY, replace)

graph combine box_Y.gph box_logY.gph, rows(1)

* OR

kdensity Y, normal saving(Y, replace)
kdensity logY, normal saving(logY, replace)

graph combine Y.gph logY.gph, rows(1)

*** Unusual and influential data

gen id=_n
scatter logY edu, mlabel(id)
scatter logY exp, mlabel(id)
scatter logY exp2, mlabel(id)

reg logY edu exp exp2

lvr2plot, mlabel(id)

dfbeta
disp 2/sqrt(2151)
scatter _dfbeta_1 _dfbeta_2 _dfbeta_3 id, ylabel(-1(.5)3) yline(.04 -.04) ///
mlabel(id id id)

*** Case by case, introduce additional peculiarities into the original dataset:

* ################################
* ### 7.2.2 Heteroskedasticity ###

* Original error without heteroskedasticity

reg logY edu exp exp2
predict res, res
gen res2=res^2
predict logY_h

// Plot the residuals versus fitted (predicted) values.

rvfplot, yline(0)
estat imtest 
estat hettest

* OR 

graph twoway scatter res logY_h, yline(0) title("No heter") saving(graph_no_heter, replace)

estat hettest			/* test using fitted values of logY */
* Indicates that heteroskedasticity is probably not a problem 
// (or at least that if it is a problem, it isnt a multiplicative function of the predicted values).
estat hettest edu		/* test using edu */
estat hettest, rhs 		/* test using right hand side variables */

* Heteroskedastic error term: variance is function of edu

gen e_a=sqrt(edu)*e

graph twoway scatter e_a edu, yline(0) title("Heter=f(edu)") saving(graph_e_a_edu, replace)

gen logY_a=7.6+ edu*.07 + exp*.012- exp2*.0005 + e_a
reg logY_a edu exp exp2
predict logY_ah
predict res_a, res
gen res_a2=res_a^2

graph twoway scatter res_a logY_ah, yline(0)  title("Heter=f(edu)") saving(graph_edu_heter, replace) 

estat hettest
estat hettest edu
reg res_a2 edu, noconstant /* Note that coefficient on edu=VAR(e)=0.1 */

* Heteroskedastic error term: variance is function of external variable

gen x=runiform() 		/* Generate variable x from a rectangular distribution over [0;1) */
gen e_b=e*(x+.01) 		/* Heteroskedastic error: variance =f(external variable x) */

graph twoway scatter e_b x, yline(0) title("Heter=f(x)") saving(graph_e_b_x, replace) 

gen logY_b=7.6+ edu*.07 + exp*.012- exp2*.0005 + e_b
reg logY_b edu exp exp2
predict logY_bh
predict res_b, res
gen res_b2=res_b^2

graph twoway scatter res_b logY_bh, yline(0)  title("Heter=f(x)") saving(graph_x_heter, replace) 

estat hettest
estat hettest, rhs
estat hettest x

graph combine graph_no_heter.gph graph_e_a_edu.gph graph_edu_heter.gph graph_e_b_x.gph graph_x_heter.gph, rows(2)

*Heteroskedasticity robust estimate of coefficient V-C matrix: sandwich estimator

reg logY_b edu exp exp2
matrix Vreg_norobust=e(V)
matrix list Vreg_norobust

reg logY_b edu exp exp2, vce(robust)		/* Robust estimation of V-C matrix */
matrix Vreg=e(V) 							/* Robust coefficients V-C matrix */
matrix list Vreg

* Vreg manually

reg logY_b edu exp exp2, mse1  	 			/* OLS w/o robust V_C; "mse1" forces mean squared error to 1 */
predict res_br, res
matrix D=e(V)					 	 		/* Non-robust V-C matrix */
matrix list D
matrix accum M = edu exp exp2 [iweight=res_b^2] /* Salami for the Sandwich; "matrix accum" accumulates cross-product matrices to form X'X */
matrix list M
matrix V=e(N)/(e(N)-e(df_m)-1) * D*M*D 		/* Sandwich X'WX */
matrix list V 


* #################################
* ### 7.2.3 Errors in variables ###

*** Case A

gen error=rnormal()				/* Measurement error*/
gen logYX=logY+.2*error			/* logY with error */

dotplot logY logYX, ny(25) saving(logY_logYX, replace) 	/* Comparative scatterplots; ny() - vertical dot density; default is ny(35) */

graph twoway scatter logY logYX logY, saving(logY_logYX_logY, replace)

reg logY edu exp exp2
reg logYX edu exp exp2 			/* See that edu coefficient is not changed, only stdandard error (increased) and R2 (descreased) */

*** Case B: Stochastic error

gen eduX=edu+2*error 			/* Education years with error */

dotplot edu eduX, ny(25) saving(edu_eduX, replace) 

graph twoway scatter edu eduX edu, saving(edu_eduX_edu, replace) 

reg logY edu exp exp2
reg logY eduX exp exp2			/* See that edu coefficient is smaller: coefficient is underestimated - 
// attenuation bias; R2 (descreased) */
corr eduX error,cov 			/* Check that the bias ~ COV(edux,error)/VAR(eduX) */

graph combine logY_logYX.gph logY_logYX_logY.gph edu_eduX.gph edu_eduX_edu.gph, rows(2)

*** Case C: systematic error =f(edu)

gen eduQ=.8*edu 					/* Education years with error */

dotplot edu eduQ, ny(25) saving(edu_eduQ, replace)

graph twoway scatter edu eduQ edu, saving(edu_eduQ_edu, replace)

reg logY edu exp exp2
reg logY eduQ exp exp2 			/* See that edu coefficient is larger; no changes in R2 */

graph combine edu_eduQ.gph edu_eduQ_edu.gph, rows(1)

locpoly logY eduX, plot(scatter logY edu) /* locpoly produces kernel-weighted local polynomial smooths; ///
plot(plot) provides a way to add other plots to the generated graph */


* #####################
* ### 7.2.4 Splines ###

/* Linear splines allow estimating the relationship between y and x as a piecewise linear function,
which is a function composed of linear segments  straight lines. 
One linear segment represents the function for values of x below x0, 
another linear segment handles values between x0 and x1, and so on. 
The linear segments are arranged so that they join at x0, x1, ... , which are called the knots. */

/* Note: A linear spline can be used to fit many functions well. 
However, a restricted cubic spline may be a better choice than a linear spline when working with a very curved function. 
When using a restricted cubic spline, one obtains a continuous smooth function that is linear before the first knot,
a piecewise cubic polynomial between adjacent knots, and linear again after the last knot. */

*** Manual creation of variables containing a spline

* Define knots

scalar X0=0
scalar X1=15
scalar X2=30

* Define range dummies

gen D1=(exp>=scalar(X1)) /* generates 1 if expression exp>=scalar(X1) is true, and 0 otherwise */
gen D2=(exp>=scalar(X2)) /* generates 1 if expression exp>=scalar(X2) is true, and 0 otherwise */

* 1st order components

gen Xf_0=exp - scalar(X0)		
gen Xf_1=D1*(exp - scalar(X1))	/* equals zero until exp reaches scalar(X1) */
gen Xf_2=D2*(exp - scalar(X2))	/* equals zero until exp reaches scalar(X2) */

graph twoway scatter  Xf_0 Xf_1 Xf_2 exp, xline(0 15 30) msymbol(oh + p) /// 
xtitle("1st order spline comp: marginal") saving(Xf_0_Xf_1_Xf_2_exp, replace) 

* 2nd order components

gen Xs_0=(exp - scalar(X0))^2
gen Xs_1=D1*(exp - scalar(X1))^2
gen Xs_2=D2*(exp - scalar(X2))^2

graph twoway scatter  Xs_0 Xs_1 Xs_2 exp, xline(0 15 30) msymbol(oh + p) /// 
xtitle("2nd order spline comp: marginal") saving(Xs_0_Xs_1_Xs_2_exp, replace) 

* 3rd order components

gen Xt_0=(exp - scalar(X0))^3
gen Xt_1=D1*(exp - scalar(X1))^3
gen Xt_2=D2*(exp - scalar(X2))^3

graph twoway scatter  Xt_0 Xt_1 Xt_2 exp, xline(0 15 30) msymbol(oh + p) /// 
xtitle("3rd order spline comp: marginal") saving(Xt_0_Xt_1_Xt_2_exp, replace) 

* Exp as exp and exp2

reg logY exp exp2 edu
gen fit0=_b[exp]*exp + _b[exp2]*exp2 + _b[_cons]

* Exp as 1st order splines

reg logY Xf_0 Xf_1 Xf_2 edu
gen fit1=_b[Xf_0]*Xf_0 + _b[Xf_1]*Xf_1 + _b[Xf_2]*Xf_2 + _b[_cons]  

* Exp as 2nd order splines

reg logY Xf_0 Xs_0 Xs_1 Xs_2 edu
gen fit2=_b[Xf_0]*Xf_0 + _b[Xs_0]*Xs_0 + _b[Xs_1]*Xs_1 + _b[Xs_2]*Xs_2 + _b[_cons]  

* Exp as 3rd order splines

reg logY Xf_0 Xs_0 Xt_0 Xt_1 Xt_2 edu
gen fit3=_b[Xf_0]*Xf_0 + _b[Xs_0]*Xs_0 + _b[Xt_0]*Xt_0 + _b[Xt_1]*Xt_1 + _b[Xt_2]*Xt_2 + _b[_cons]  

* Fitted splines

*** Stata makes splines: marginal version
label variable fit0 "Standard fit"
label variable fit1 "1st order spline"
label variable fit2 "2nd order spline"
label variable fit3 "3rd order spline"

graph twoway scatter fit0 fit1 fit2 fit3 exp, xline(0 15 30) /// 
xtitle("Splines Fits") saving(fit0_fit1_fit2_fit3, replace) 

* "mkspline" creates variables containing a linear spline or a restricted cubic spline of oldvar.

/* "marginal" option specifies that the new variables is constructed so that, when used in estimation,
the coefficients on these variables represent the change in the slope from the preceding interval.  
The default is to construct the variables so that, when used in estimation, 
the coefficients measure the slopes for the interval.

A key advantage of "marginal" usage is that it makes it possible to test whether the change in slope is significant, 
i.e. if the effect of respective variable from the following period is not significant 
then the effect of respective variable does not change after the break point.

Specifying "marginal" changes only the interpretation of the coeficients; 
the same model is fit in either case. */

mkspline Qf_0 15 Qf_1 30 Qf_2 = exp, marginal   				/* 1st order components */
for M in new Qs_0 Qs_1 Qs_2 \ N in var Qf_0 Qf_1 Qf_2: g M=N^2  /* 2nd order components */
// ->  g Qs_0=Qf_0^2
// ->  g Qs_1=Qf_1^2
// ->  g Qs_2=Qf_2^2

for M in new Qt_0 Qt_1 Qt_2 \ N in var Qf_0 Qf_1 Qf_2: g M=N^3  /* 3rd order components */
// ->  g Qt_0=Qf_0^3
// ->  g Qt_1=Qf_1^3
// ->  g Qt_2=Qf_2^3

graph twoway scatter Qf_1 Xf_1 exp, msymbol(+ p)
graph twoway scatter Qs_1 Xs_1 exp, msymbol(+ p)
graph twoway scatter Qt_1 Xt_1 exp, msymbol(+ p)
graph twoway scatter Qf_0 Qf_1 Qf_2 exp, xline(0 15 30) msymbol(oh + p)
reg logY Qf_0 Qf_1 Qf_2 edu

* Stata makes splines: standard version

mkspline Wf_0 15 Wf_1 30 Wf_2 = exp
graph twoway scatter Wf_0 Wf_1 Wf_2 exp, xline(0 15 30) msymbol(oh + p) saving(Wf_0_Wf_1_Wf_2, replace) xtitle("1st order spline comp:")
reg logY Wf_0 Wf_1 Wf_2 edu
graph combine Xf_0_Xf_1_Xf_2_exp.gph fit0_fit1_fit2_fit3.gph Wf_0_Wf_1_Wf_2.gph

reg logY Wf_0 Wf_1 Wf_2 edu
reg logY Xf_0 Xf_1 Xf_2 edu

gen diff1 = -0.012682 - 0.0066439
disp diff1
gen diff2 = -.0170417 - (-0.012682)
disp diff2


* ####################################
* ### 7.2.5 Instrumental variables ###

* Griliches (1977) recognized that endogeneity of schooling decisions is a serious issue which might prevented economists 
// to uncover the true causal effect of education on earnings. 
// A wide range of empirical papers using instrumental variable (IV) techniques have been published. 
// A large segment of this literature is based on institutional features of the education system.

mat miv=(3,12,20,0) /*matrix of means of RHS vars: IV, edu, exp, error*/
mat civ=(2,.5,.01,0\.5,5,-.6,.5 \.01,-.6,119,0 \0,.5,0,.1) /*covariance matrix of RHS vars */ 
mat list miv		/* displays matrix of means */
mat list civ		/* displays covariance matrix */

* edu is correlated with error, while IV is not; IV is correlated with edu

* Draw from a multivariate normal distribution with desired means and covariance matrix.

drawnorm IV EDU EXP E, means(miv) cov(civ)   
gen EXP2=EXP^2
gen LOGY=7.6+ EDU*.07 + EXP*.012 - EXP2*.0005 + E

graph matrix LOGY IV EDU EXP EXP2 E, msymbol(oh)

corr LOGY IV EDU EXP EXP2 E

* edu is much higher correlated with error that IV

* OLS (biased estimates)

reg LOGY EDU EXP EXP2

* 2SLS - IV estimation

ivreg LOGY (EDU=IV) EXP EXP2	/* Stata command */

reg EDU IV EXP EXP2
predict EDU_hat
reg LOGY EDU_hat EXP EXP2		/* Own IV */


* ####################################
* ### 7.2.6 Wage gap decomposition ###

*** Methods and Formulas

/* Let y1 and y2 be the means of the dependent variable Y, 
x1 and x2 the row vectors of the means of the explanatory variables X1,...,Xk, 
and b1 and b2 the column vectors of the coefficient
for group 1 (high) and group 2 (low).  
The raw differential y1-y2 may then be expressed as

    R = y1-y2 = (x1-x2)b2 + x2(b1-b2) + (x1-x2)(b1-b2) = E + C + CE

i.e., R is decomposed into a part due to differences in endowments (E), 
a part due to differences in coefficients (including the intercept) (C), 
and a part due to interaction between coefficients and endowments (CE). 

Depending on the model which is assumed to be non-discriminating, 
these terms may be used to determine the "unexplained" (U; discrimination) 
and the "explained" (V) part of the differential 
(the question is how to allocate the interaction term CE). 

* # (1,2) Two Oaxaca decompositions

Oaxaca (1973) proposed to assume either the low group model or the high group model as non-discriminating, 
which leads to U=C+CE and V=E or U=C and V=E+CE, respectively. 
More generally the decomposition may be written as

    y1-y2 = (x1-x2)[D*b1+(I-D)*b2] + [x1*(I-D)+x2*D](b1-b2)

where I is a identity matrix and D is a diagonal matrix of weights. 
In the two cases proposed by Oaxaca (1973) D is a nullmatrix or equals I, 
respectively (D=I is also what Blinder 1973 suggested). 

-> D=0
y1-y2 = (x1-x2)[D*b1+(I-D)*b2] + [x1*(I-D)+x2*D](b1-b2) = (x1-x2)[b2] + [x1](b1-b2) = E + (C + CE)

-> D=1
y1-y2 = (x1-x2)[D*b1+(I-D)*b2] + [x1*(I-D)+x2*D](b1-b2) = (x1-x2)[b1] + [x2](b1-b2) = (E + CE) + C

* # (3) Reimers' decomposition

Reimers (1983) proposed to use the mean coefficients between the low and the high model, 
i.e. the diagonal elements of D equal 0.5, 

* # (4) Cotton' decomposition

Cotton (1988) proposed to weight the coefficients by group size, 
i.e. the diagonal elements of D equal fH, 
where fH is the relative proportion of subjects in the high group (or sum of weights, if weights are applied). 

* # (5) Neumark' decomposition

Finally, Neumark (1988) proposed to estimate a pooled model over both groups, which leads to D=diag(bP-b2)*diag(b1-b2)^-1 or

    y1-y2 = (x1-x2)bP + [x1(b1-bP)+x2(bP-b2)]

where bP is the column vector of the coefficients in the pooled model.

"decompose" calculates and displays R, E, C, CE, as well as U and V according to the methods described.
The coefficient vectors are taken from "e(b)" returned by the estimation commands, 
the means of the explanatory variables and group sizes are calculated for "e(sample)" using summarize (weighted if necessary). */

clear

ssc install decompose

* DEFINITIONS

mat m_W=(12,18,0)	/* matrix of means of RHS vars for Whites: edu, exp, error */
mat c_W=(5,-.6, 0 \ -.6,119,0 \ 0,0,.1) /*covariance matrix of RHS vars for Whites */ 
mat list m_W		/* displays matrix of means */
mat list c_W		/* displays covariance matrix */

mat m_B=(8,23,0)	/* matrix of means of RHS vars for Blacks: edu, exp, error */
mat c_B=(5,-.6, 0 \ -.6,119,0 \ 0,0,.1) /*covariance matrix of RHS vars for Blacks */ 
mat list m_B		/* displays matrix of means */
mat list c_B		/* displays covariance matrix */

drop _all			/* Drop all observations and variables */

* Draw a sample of 2000 OBS for Whites

set seed 10000
set obs 2000
gen black=0
drawnorm edu exp e, means(m_W) cov(c_W)
save Whites1.dta, replace

drop _all

* Draw a sample of 1000 OBS for Blacks

set seed 20000
set obs 1000
gen black=1
drawnorm edu exp e ,means(m_B) cov(c_B)

append using Whites1.dta

drop if (exp<0 | exp>40)    /* Dropping observations with extreme values of exp */
gen exp2=exp^2
gen logY=7.6+ edu*.07 + exp*.012  + e if black==0		/* Generate log of earnings for Whites */
replace logY=4.6+ edu*.04 + exp*.012 + e if black==1	/* Generate log of earnings for Blacks */

table black, contents(mean logY mean edu mean exp mean e)

decompose logY edu exp, by(black) detail estimates /* "detail" additionally displays decomposition results for variables. 
estimates additionally displays a table of regressions coefficients and means. */

* # Summary of decomposition results

* ## The first block

/* The first block of output reports the mean values of logY for the two groups, 
and the difference between them. 
It then shows the contribution attributable to the gaps in endowments (E), 
the coefficients (C), and the interaction (CE).
The gap in coefficients accounts for the great bulk of the gap in outcomes. */

/* The earnings gap is thus divided into three parts. 
# The first part (due to endowments) reflects the mean increase in Blacks' earnings 
if they had the same characteristics as Whites. 
# The second term (due to coefficients) quantifies the change in Blacks' earnings 
when applying the Whites' coeficients to the Blacks' characteristics. 
# The third part (due to interaction) is the interaction term that measures 
the simultaneous effect of differences in endowments and coeficients. */

* ## The second block

/* The second block of output shows how the explained and unexplained portions 
of the outcome gap vary depending on the decomposition used.
The first and second columns correspond to the Oaxaca decomposition, 
where D = 0 and D = I, respectively. 
The third and fourth columns correspond to Reimers and Cottons decompositions, 
where the diagonal of D equals 0.5 and fH = 0.670 (in our case), respectively. */

tabulate black
display (1859)/2774

/* The final column labeled * is Neumarks decomposition. 

* # Decomposition results for variables

/* The third block of output allows the user to see how far gaps in individual xs contribute 
to the overall explained gap.
It is the gap in education (edu) that accounts for the bulk of the explained gap. */

* # Coefficients, Means and Predictions

/* The fourth and final block of output gives the coefficient estimates, means, and predictions 
for each x for each group, the high group in this case being the Whites and the low group being the Blacks. */

/* For the first Oaxaca decomposition columns 2 and 3 of table "Decomposition results for variables"
allow us to identify how the gap in each of the betas contributes to the overall unexplained gap. 

For the other decompositions, the contributions of the individual betas can be found 
by taking the group difference in the variable specific predictions given in table 
"Coefficients, Means and Predictions" and subtracting the explained part 
given in table "Decomposition results for variables" from this. */
